@@ -86,7 +86,7 @@ let mut g = das_grid::Grid::new(5, 5, 0);
 // Print with special {:?} to see the contents of the grid
 println!("{:?}", g);
 // outputs:
-// Grid { width: 5, height: 5, cells: [
+// Grid { rows: 5, cols: 5, cells: [
 //  0 (x: 0 y: 0) 0 (x: 1 y: 0) 0 (x: 2 y: 0) 0 (x: 3 y: 0) 0 (x: 4 y: 0)
 //  0 (x: 0 y: 1) 0 (x: 1 y: 1) 0 (x: 2 y: 1) 0 (x: 3 y: 1) 0 (x: 4 y: 1)
 //  0 (x: 0 y: 2) 0 (x: 1 y: 2) 0 (x: 2 y: 2) 0 (x: 3 y: 2) 0 (x: 4 y: 2)
@@ -145,11 +145,11 @@ pub enum GridErr {
 impl fmt::Display for GridErr {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            GridErr::OutOfGrid => write!(f, "value is out of the grid width and height"),
+            GridErr::OutOfGrid => write!(f, "value is out of the grid rows and cols"),
             GridErr::RuleFailed => write!(f, "failed to meet the rule requirements"),
             GridErr::SubgridOverflow => write!(
                 f,
-                "the subgrid height or width is greater than the parent grid"
+                "the subgrid cols or rows is greater than the parent grid"
             ),
         }
     }
@@ -157,10 +157,10 @@ impl fmt::Display for GridErr {
 
 /// Represents the possible direction to move
 ///
-/// MoveDirection::Right (1, 0)
-/// MoveDirection::Left (-1, 0)
-/// MoveDirection::Up (0, -1)
-/// MoveDirection::Down (0, 1)
+/// MoveDirection::Right
+/// MoveDirection::Left
+/// MoveDirection::Up
+/// MoveDirection::Down
 #[derive(Debug, PartialEq, Eq)]
 pub enum MoveDirection {
     Right,
@@ -169,17 +169,17 @@ pub enum MoveDirection {
     Down,
 }
 
-/// Represent move to right position on Das Grid (1, 0)
-pub const MOVE_RIGHT: (i32, i32) = (1, 0);
+/// Represent move to right position on Das Grid (0, 1)
+pub const MOVE_RIGHT: (i32, i32) = (0, 1);
 
-/// Represent move to left position on Das Grid (-1, 0)
-pub const MOVE_LEFT: (i32, i32) = (-1, 0);
+/// Represent move to left position on Das Grid (0, -1)
+pub const MOVE_LEFT: (i32, i32) = (0, -1);
 
-/// Represent move to up position on Das Grid (0, -1)
-pub const MOVE_UP: (i32, i32) = (0, -1);
+/// Represent move to up position on Das Grid (-1, 0)
+pub const MOVE_UP: (i32, i32) = (-1, 0);
 
-/// Represent move to down position on Das Grid (0, 1)
-pub const MOVE_DOWN: (i32, i32) = (0, 1);
+/// Represent move to down position on Das Grid (1, 0)
+pub const MOVE_DOWN: (i32, i32) = (1, 0);
 
 /// Stores the grid values and the cells
 /// The grid itself representation is a flatten vector which is transformed
@@ -204,8 +204,8 @@ pub const MOVE_DOWN: (i32, i32) = (0, 1);
 /// ```
 
 pub struct Grid<T: Copy + Clone> {
-    pub(crate) width: i32,
-    pub(crate) height: i32,
+    pub(crate) rows: i32,
+    pub(crate) cols: i32,
     pub(crate) initial_value: T,
     pub(crate) cells: Vec<T>,
 }
@@ -217,20 +217,50 @@ impl<T: Copy + Clone> Grid<T> {
     /// let grid = das_grid::Grid::new(2, 2, 1);
     /// assert_eq!(grid.size(), 4);
     /// ```
-    pub fn new(width: i32, height: i32, value: T) -> Self
+    pub fn new(rows: i32, cols: i32, value: T) -> Self
     where
-        T: Clone + Copy + Display,
+        T: Clone + Copy,
     {
-        if (width * height) == 0 {
+        if (rows * cols) == 0 {
             panic!("0x0 grid is forbidden")
         }
 
         let initial_value = value;
-        let cells = vec![value; (width * height) as usize];
+        let cells = vec![value; (rows * cols) as usize];
 
         Self {
-            width,
-            height,
+            rows,
+            cols,
+            initial_value,
+            cells,
+        }
+    }
+
+    /// Creates a grid from a given vector with quadratic size
+    /// For example this will generate a 2x2 grid
+    /// ```.rust
+    /// let mut grid = das_grid::Grid::new_from_vector(2, 2, vec![1, 2, 3, 4]);
+    /// assert_eq!(grid.size(), 4);
+    /// ```
+    pub fn new_from_vector(rows: i32, cols: i32, vec: Vec<T>) -> Self {
+        if vec.len() % 2 != 0 {
+            panic!("The vector isn't multiple of 2");
+        }
+
+        if vec.len() == 0 {
+            panic!("0x0 grid is forbidden")
+        }
+
+        if rows * cols != vec.len() as i32 {
+            panic!("cols and rows should be same vector size")
+        }
+
+        let initial_value = vec.first().unwrap().clone();
+        let cells = vec.to_vec();
+
+        Self {
+            rows,
+            cols,
             initial_value,
             cells,
         }
@@ -270,6 +300,35 @@ impl<T: Copy + Clone> Grid<T> {
         Ok(())
     }
 
+    /// Creates the a new grid which is a snapshot of the main grid on the given position and size
+    ///
+    /// If the sub grid is greater than the main grid it return an error of GridErr::SubgridOverflow
+    ///
+    /// Or if the dest x, y grid is out of bounds it return error GridErr::OutOfGrid
+    ///
+    /// ```.rust
+    /// let mut grid = das_grid::Grid::new_from_vector(4, 4, (1..=16).collect());
+    /// let sub_grid = grid.get_subgrid((2, 2), 2, 2).unwrap();
+    /// assert_eq!(sub_grid.get_flatten_grid(), vec![11, 12, 15, 16]);
+    /// ```
+    pub fn get_subgrid(&self, index: (i32, i32), rows: i32, cols: i32) -> Result<Grid<T>, GridErr> {
+        self.check_grid_bounds(index)?;
+        let mut sub_grid = Grid::new(rows, cols, self.initial_value);
+        self.check_grid_overflow(&sub_grid)?;
+
+        for sub_index in sub_grid.enumerate() {
+            let dest = (index.0 + sub_index.0, index.1 + sub_index.1);
+            if let Ok(subv) = self.get(dest) {
+                match sub_grid.set(sub_index, &subv) {
+                    Ok(_) => (),
+                    _ => (),
+                }
+            }
+        }
+
+        Ok(sub_grid)
+    }
+
     /// Stamps the subgrid into the destiny grid, merging both
     /// Only if no rule return error
     ///
@@ -332,11 +391,11 @@ impl<T: Copy + Clone> Grid<T> {
 
     // Check if subgrid isn't bigger than the destiny grid
     fn check_grid_overflow(&self, sub_grid: &Grid<T>) -> Result<(), GridErr> {
-        if sub_grid.height > self.height {
+        if sub_grid.cols > self.cols {
             return Err(GridErr::SubgridOverflow);
         }
 
-        if sub_grid.width > self.width {
+        if sub_grid.rows > self.rows {
             return Err(GridErr::SubgridOverflow);
         }
 
@@ -347,11 +406,11 @@ impl<T: Copy + Clone> Grid<T> {
     fn check_grid_bounds(&self, index: (i32, i32)) -> Result<(), GridErr> {
         let (x, y) = index;
 
-        if x < 0 || x >= self.width {
+        if x < 0 || x >= self.rows {
             return Err(GridErr::OutOfGrid);
         }
 
-        if y < 0 || y >= self.height {
+        if y < 0 || y >= self.cols {
             return Err(GridErr::OutOfGrid);
         }
 
@@ -375,7 +434,7 @@ impl<T: Copy + Clone> Grid<T> {
 
         self.check_grid_bounds(index)?;
 
-        if let Some(cell) = self.cells.get_mut((x + (y * self.width)) as usize) {
+        if let Some(cell) = self.cells.get_mut((x * self.rows + y) as usize) {
             *cell = *value;
         }
 
@@ -435,7 +494,7 @@ impl<T: Copy + Clone> Grid<T> {
 
         self.check_grid_bounds(index)?;
 
-        Ok(self.cells.get_mut((x + (y * self.width)) as usize).unwrap())
+        Ok(self.cells.get_mut((x * self.rows + y) as usize).unwrap())
     }
 
     /// Gets a give value to the position (x, y)
@@ -453,7 +512,7 @@ impl<T: Copy + Clone> Grid<T> {
 
         self.check_grid_bounds(index)?;
 
-        Ok(self.cells.get((x + (y * self.width)) as usize).unwrap())
+        Ok(self.cells.get((x * self.rows + y) as usize).unwrap())
     }
 
     /// Moves a given value from position (x, y) to destiny position (x, y)
@@ -475,13 +534,63 @@ impl<T: Copy + Clone> Grid<T> {
         Ok(())
     }
 
+    /// Moves a given value from position (x, y) to destiny position (x, y)
+    /// Only if no rule return error
+    ///
+    /// Be careful if the value is out of the bounds of grid it will return an error
+    /// with the type of GridErr::OutOfGrid
+    ///
+    /// And if a rule some rule failed it will return GridErr::RuleFailed
+    ///
+    /// ```.rust
+    /// let mut grid = das_grid::Grid::new(2, 2, 0);
+    /// assert!(grid.set((0, 1), &1).is_ok());
+    ///
+    /// let rule_not_1 = |_: (i32, i32), value: &i32| -> Result<(), das_grid::GridErr> {
+    ///     if *value == 1 {
+    ///         return Err(das_grid::GridErr::RuleFailed);
+    ///     }
+    ///     Ok(())
+    /// };
+    ///
+    /// assert!(
+    ///     grid.mov_with_rules((0, 0), (0, 1), vec![rule_not_1])
+    ///         .err()
+    ///         .unwrap()
+    ///         == das_grid::GridErr::RuleFailed
+    /// );
+    /// ```
+    pub fn mov_with_rules<R>(
+        &mut self,
+        index: (i32, i32),
+        dest: (i32, i32),
+        rules: Vec<R>,
+    ) -> Result<(), GridErr>
+    where
+        R: Fn((i32, i32), &T) -> Result<(), GridErr>,
+    {
+        self.check_grid_bounds(index)?;
+        self.check_grid_bounds(dest)?;
+        let prev = *self.get_mut(index).unwrap();
+
+        let destv = self.get(dest)?;
+        for rule in rules {
+            rule(dest, destv)?;
+        }
+
+        self.set(index, &self.initial_value.clone())?;
+        self.set(dest, &prev)?;
+
+        Ok(())
+    }
+
     /// Moves a given value from position (x, y) to another position based on the direction
     ///
     /// The directions can be Left, Right, Top, Down:
-    /// * DasGrid::MoveDirection::Left, translates to (-1, 0)
-    /// * DasGrid::MoveDirection::Right, translates to (1, 0)
-    /// * DasGrid::MoveDirection::Top, translates to (0, -1)
-    /// * DasGrid::MoveDirection::Down, translates to (0, 1)
+    /// * DasGrid::MoveDirection::Left, translates to (0, -1)
+    /// * DasGrid::MoveDirection::Right, translates to (0, 1)
+    /// * DasGrid::MoveDirection::Top, translates to (-1, 0)
+    /// * DasGrid::MoveDirection::Down, translates to (1, 0)
     ///
     /// Be careful if the value is out of the bounds of grid it will return an error
     /// with the type of GridErr::OutOfGrid
@@ -511,6 +620,68 @@ impl<T: Copy + Clone> Grid<T> {
         Ok(())
     }
 
+    /// Moves a given value from position (x, y) to another position based on the direction
+    /// Only if no rule return error
+    ///
+    /// if the dest x, y grid is out of bounds it return error GridErr::OutOfGrid
+    ///
+    /// And if a rule some rule failed it will return GridErr::RuleFailed
+    ///
+    /// The directions can be Left, Right, Top, Down:
+    /// * DasGrid::MoveDirection::Left, translates to (0, -1)
+    /// * DasGrid::MoveDirection::Right, translates to (0, 1)
+    /// * DasGrid::MoveDirection::Top, translates to (-1, 0)
+    /// * DasGrid::MoveDirection::Down, translates to (1, 0)
+    ///
+    /// Be careful if the value is out of the bounds of grid it will return an error
+    /// with the type of GridErr::OutOfGrid
+    ///
+    /// ```.rust
+    /// let mut g = das_grid::Grid::new(2, 2, 0);
+    /// g.set((0, 1), &1);
+    /// let rule_not_1 = |_: (i32, i32), value: &i32| -> Result<(), das_grid::GridErr> {
+    ///     if *value == 1 {
+    ///         return Err(das_grid::GridErr::RuleFailed);
+    ///     }
+    ///     Ok(())
+    /// };
+    /// let ret = g.mov_to_with_rules((0, 0), das_grid::MoveDirection::Right, vec![rule_not_1]);
+    /// assert!(ret.is_err());
+    /// ```
+    pub fn mov_to_with_rules<R>(
+        &mut self,
+        index: (i32, i32),
+        direction: MoveDirection,
+        rules: Vec<R>,
+    ) -> Result<(), GridErr>
+    where
+        R: Fn((i32, i32), &T) -> Result<(), GridErr>,
+    {
+        let (x, y) = index;
+        self.check_grid_bounds(index)?;
+
+        let (xx, yy) = match direction {
+            MoveDirection::Up => MOVE_UP,
+            MoveDirection::Down => MOVE_DOWN,
+            MoveDirection::Left => MOVE_LEFT,
+            MoveDirection::Right => MOVE_RIGHT,
+        };
+
+        let dest = (x + xx, y + yy);
+        self.check_grid_bounds(dest)?;
+
+        let destv = self.get(dest)?;
+        for rule in rules {
+            rule(dest, destv)?;
+        }
+
+        let prev = *self.get_mut(index).unwrap();
+        self.set(index, &self.initial_value.clone())?;
+        self.set(dest, &prev)?;
+
+        Ok(())
+    }
+
     /// Get the size of grid based on cells length
     ///
     /// For instance a 10x10 grid will return the size of 100
@@ -523,22 +694,22 @@ impl<T: Copy + Clone> Grid<T> {
         self.cells.len()
     }
 
-    /// The width of the grid
+    /// The rows of the grid
     /// ```.rust
     /// let mut grid = das_grid::Grid::new(3, 2, 1);
-    /// assert_eq!(grid.width(), 3);
+    /// assert_eq!(grid.rows(), 3);
     /// ```
-    pub fn width(&self) -> i32 {
-        self.width
+    pub fn rows(&self) -> i32 {
+        self.rows
     }
 
-    /// The height of the grid
+    /// The cols of the grid
     /// ```.rust
     /// let mut grid = das_grid::Grid::new(3, 2, 1);
-    /// assert_eq!(grid.height(), 2);
+    /// assert_eq!(grid.cols(), 2);
     /// ```
-    pub fn height(&self) -> i32 {
-        self.height
+    pub fn cols(&self) -> i32 {
+        self.cols
     }
 
     /// Returns the grid as a tuple of (x, y)
@@ -556,7 +727,7 @@ impl<T: Copy + Clone> Grid<T> {
             .iter()
             .enumerate()
             .map(|(idx, _)| {
-                if idx as i32 % self.width() == 0 && idx > 1 {
+                if idx as i32 % self.rows() == 0 && idx > 1 {
                     x = 0;
                     y += 1;
                 }
@@ -566,6 +737,103 @@ impl<T: Copy + Clone> Grid<T> {
             })
             .collect::<Vec<_>>()
     }
+
+    /// Returns the type vector with the values from the col
+    ///
+    /// If the col idx is wrong it can return the error GridErr::OutOfGrid
+    ///
+    /// ```.rust
+    /// let mut g = das_grid::Grid::new_from_vector(2, 2, vec![1, 2, 3, 4]);
+    /// let col = g.get_col(1).unwrap();
+    /// assert_eq!(col, vec![2, 4]);
+    /// ```
+    pub fn get_col(&self, col_idx: i32) -> Result<Vec<T>, GridErr> {
+        let mut vec_result: Vec<T> = vec![];
+        for idx in (0..self.cols).into_iter() {
+            let v = self.get((idx, col_idx))?;
+            vec_result.push(*v);
+        }
+        Ok(vec_result)
+    }
+
+    /// Returns the type vector with the values from the row
+    ///
+    /// If the row idx is wrong it can return the error GridErr::OutOfGrid
+    ///
+    /// ```.rust
+    /// let mut g = das_grid::Grid::new_from_vector(2, 2, vec![1, 2, 3, 4]);
+    /// let row = g.get_row(1).unwrap();
+    /// assert_eq!(row, vec![3, 4]);
+    /// ```
+    pub fn get_row(&self, row_idx: i32) -> Result<Vec<T>, GridErr> {
+        let mut vec_result: Vec<T> = vec![];
+        for idx in (0..self.rows).into_iter() {
+            let v = self.get((row_idx, idx))?;
+            vec_result.push(*v);
+        }
+        Ok(vec_result)
+    }
+
+    /// Returns a clone of the internal representation of the grid
+    ///
+    /// ```.rust
+    /// let mut g = das_grid::Grid::new_from_vector(2, 2, vec![1, 2, 3, 4]);
+    /// assert_eq!(g.get_flatten_grid(), vec![1,2,3,4]);
+    /// ```
+    pub fn get_flatten_grid(&self) -> Vec<T> {
+        self.cells.clone()
+    }
+
+    /// Fill the grid with the given value
+    ///
+    /// ```.rust
+    ///
+    /// ```
+    pub fn fill_grid(&mut self, value: T) {
+        self.cells.fill(value);
+    }
+
+    #[allow(dead_code)]
+    pub(crate) fn debug(&self)
+    where
+        T: std::fmt::Display,
+    {
+        println!("{:?}", self)
+    }
+
+    /// Fills the certain area of the grid with a given value
+    ///
+    /// If the area is greater than the main grid it return an error of GridErr::SubgridOverflow
+    ///
+    /// ```.rust
+    /// let mut grid = das_grid::Grid::new_from_vector(4, 4, (1..=16).collect());
+    /// grid.fill_subgrid((1, 1), 2, 2, &0);
+    /// assert!(grid.get((1, 1)).unwrap() == &0);
+    /// assert!(grid.get((1, 2)).unwrap() == &0);
+    /// assert!(grid.get((2, 1)).unwrap() == &0);
+    /// assert!(grid.get((2, 2)).unwrap() == &0);
+    /// ```
+    pub fn fill_subgrid(
+        &mut self,
+        index: (i32, i32),
+        rows: i32,
+        cols: i32,
+        value: &T,
+    ) -> Result<Grid<T>, GridErr> {
+        self.check_grid_bounds(index)?;
+        let sub_grid = Grid::new(rows, cols, self.initial_value);
+        self.check_grid_overflow(&sub_grid)?;
+
+        for sub_index in sub_grid.enumerate() {
+            let dest = (index.0 + sub_index.0, index.1 + sub_index.1);
+            match self.set(dest, value) {
+                Ok(_) => (),
+                _ => (),
+            }
+        }
+
+        Ok(sub_grid)
+    }
 }
 
 impl<'a, T: Copy + Clone> IntoIterator for &'a Grid<T> {
@@ -588,8 +856,8 @@ impl<T: Copy + Clone> fmt::Display for Grid<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
             f,
-            "Grid {{ width: {}, height: {}, cells: [...] }}",
-            self.width, self.height
+            "Grid {{ rows: {}, cols: {}, cells: [...] }}",
+            self.rows, self.cols
         )
     }
 }
@@ -600,19 +868,19 @@ impl<T: Copy + Clone + Display> fmt::Debug for Grid<T> {
 
         let mut pos = (0, 0);
         for (idx, cell) in self.cells.iter().enumerate() {
-            if idx as i32 % self.width == 0 && idx > 0 {
-                pos.0 = 0;
-                pos.1 += 1;
+            if idx as i32 % self.cols == 0 && idx > 0 {
+                pos.1 = 0;
+                pos.0 += 1;
                 cell_str += "\n";
             }
             cell_str.push_str(&format!("\t{:3} (x: {} y: {})", cell, pos.0, pos.1));
-            pos.0 += 1
+            pos.1 += 1
         }
 
         write!(
             f,
-            "Grid {{ width: {}, height: {}, cells: [\n{}\n] }}",
-            self.width, self.height, cell_str,
+            "Grid {{ rows: {}, cols: {}, cells: [\n{}\n] }}",
+            self.rows, self.cols, cell_str,
         )
     }
 }
