@@ -98,13 +98,13 @@ fn test_move_to() {
     g.set((0, 0), &1);
 
     let ret = g.mov_to((0, 0), MoveDirection::Right);
-    assert!(g.get((1, 0)).unwrap() == &1);
+    assert!(g.get((0, 1)).unwrap() == &1);
     assert!(ret.is_ok());
 
-    let ret = g.mov_to((1, 0), MoveDirection::Right);
+    let ret = g.mov_to((0, 1), MoveDirection::Right);
     assert!(ret.unwrap_err() == GridErr::OutOfGrid);
 
-    let ret = g.mov_to((1, 0), MoveDirection::Left);
+    let ret = g.mov_to((0, 1), MoveDirection::Left);
     assert!(g.get((0, 0)).unwrap() == &1);
 
     let ret = g.mov_to((0, 0), MoveDirection::Left);
@@ -116,10 +116,10 @@ fn test_move_to() {
     let ret = g.mov_to((0, 0), MoveDirection::Down);
     assert!(ret.is_ok());
 
-    let ret = g.mov_to((0, 1), MoveDirection::Down);
+    let ret = g.mov_to((1, 0), MoveDirection::Down);
     assert!(ret.unwrap_err() == GridErr::OutOfGrid);
 
-    let ret = g.mov_to((0, 1), MoveDirection::Right);
+    let ret = g.mov_to((1, 0), MoveDirection::Right);
     assert!(ret.is_ok());
 
     let ret = g.mov_to((1, 1), MoveDirection::Right);
@@ -196,3 +196,60 @@ fn test_stamp_subgrid_with_rules_1() {
         .stamp_subgrid_with_rules((5, 5), sub_grid, vec![rule_not_1])
         .is_err());
 }
+
+#[test]
+fn test_get_row() {
+    let mut g = Grid::new_from_vector(2, 2, vec![1, 2, 3, 4]);
+    let row = g.get_row(1).unwrap();
+    assert_eq!(row, vec![3, 4]);
+    let col = g.get_col(1).unwrap();
+    assert_eq!(col, vec![2, 4]);
+}
+
+#[test]
+fn test_new_from_vec() {
+    let mut g = Grid::new_from_vector(2, 2, vec![1, 2, 3, 4]);
+    assert_eq!(g.size(), 4);
+}
+
+#[test]
+fn test_get_subgrid() {
+    let mut grid = Grid::new_from_vector(4, 4, (1..=16).collect());
+    let sub_grid = grid.get_subgrid((2, 2), 2, 2).unwrap();
+    assert_eq!(sub_grid.get_flatten_grid(), vec![11, 12, 15, 16]);
+}
+
+#[test]
+fn test_mov_to_with_rules() {
+    let mut g = Grid::new(2, 2, 0);
+    g.set((0, 1), &1);
+
+    let rule_not_1 = |_: (i32, i32), value: &i32| -> Result<(), GridErr> {
+        if *value == 1 {
+            return Err(GridErr::RuleFailed);
+        }
+        Ok(())
+    };
+
+    let ret = g.mov_to_with_rules((0, 0), MoveDirection::Right, vec![rule_not_1]);
+    assert!(ret.is_err());
+}
+
+#[test]
+fn test_fill_subgrid() {
+    let mut grid = Grid::new_from_vector(4, 4, (1..=16).collect());
+    grid.fill_subgrid((1, 1), 2, 2, &0);
+    assert!(grid.get((1, 1)).unwrap() == &0);
+    assert!(grid.get((1, 2)).unwrap() == &0);
+    assert!(grid.get((2, 1)).unwrap() == &0);
+    assert!(grid.get((2, 2)).unwrap() == &0);
+}
+
+// #[test]
+// fn test_generic() {
+//     let mut g = Grid::new(3, 3, 0);
+//     g.set((0, 1), &1);
+//     g.debug();
+//     println!("{:?}", g.mov_to((0, 1), MoveDirection::Right));
+//     g.debug();
+// }
